﻿using ELTE.TicTacToeGame.Persistence;
using ELTE.TicTacToeGame.Model;
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ELTE.TicTacToeGame.View
{
    /// <summary>
    /// Tic-Tac-Toe ablak típusa.
    /// </summary>
    public partial class TicTacToeForm : Form
    {
        #region Private fields

        private TicTacToeModel model; // játék
        private Button[,] buttonBoard; // gombrács

        #endregion

        #region Constructors

        /// <summary>
        /// Tic-Tac-Toe ablak létrehozása.
        /// </summary>
        public TicTacToeForm()
        {
            this.InitializeComponent();

            // modell létrehozása és eseménykezelők társítása
            this.model = new ConnectFourModel(10);
            this.model.FieldChanged += new EventHandler<FieldChangedEventArgs>(Model_FieldChanged);
            this.model.GameOver += new EventHandler(Model_GameOver);
            this.model.GameWon += new EventHandler<GameWonEventArgs>(Model_GameWon);

            // új játék indítása
            this.model.NewGame();

            // játéktábla létrehozása
            this.GenerateBoard();
        }

        #endregion

        #region Private methods

        /// <summary>
        /// Tábla létrehozása.
        /// </summary>
        private void GenerateBoard()
        {
            this.buttonBoard = new Button[this.model.BoardWidth, this.model.BoardHeight];
            for (Int32 i = 0; i < this.model.BoardWidth; i++)
                for (Int32 j = 0; j < this.model.BoardHeight; j++)
                {
                    this.buttonBoard[i, j] = new Button();
                    this.buttonBoard[i, j].Font = new Font(FontFamily.GenericSansSerif, this.tableLayoutPanel.Height / this.model.BoardHeight / 2, FontStyle.Bold); // betűtípus
                    this.buttonBoard[i, j].TabIndex = 100 + i * this.model.BoardWidth + j; // a gomb számát a TabIndex-ben tároljuk
                    this.buttonBoard[i, j].Dock = DockStyle.Fill; // kitölti a helyet
                    this.buttonBoard[i, j].BackColor = Color.White;
                    this.buttonBoard[i, j].Click += new EventHandler(ButtonBoard_Click);
                    // közös eseménykezelő hozzárendelése minden gombhoz

                    this.tableLayoutPanel.Controls.Add(this.buttonBoard[i, j], i, j);
                    // felevesszük az ablakra a gombot
                }

            this.tableLayoutPanel.RowCount = this.model.BoardHeight;
            this.tableLayoutPanel.ColumnCount = this.model.BoardWidth;

            // létre kell hozni a megfelelő stípusokat a soroknak és oszlopoknak
            this.tableLayoutPanel.RowStyles.Clear();
            this.tableLayoutPanel.ColumnStyles.Clear();

            for (Int32 i = 0; i < this.model.BoardWidth; i++)
            {
                this.tableLayoutPanel.RowStyles.Add(new RowStyle(SizeType.Percent, 1.0F / this.model.BoardWidth));
                // a sorok és oszlopok egyenlően, arányosan méreteződnek
            }
            for (Int32 j = 0; j < this.model.BoardHeight; j++)
            {
                this.tableLayoutPanel.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 1.0F / this.model.BoardHeight));
            }
        }

        /// <summary>
        /// Tábla frissítése.
        /// </summary>
        private void UpdateBoard()
        {
            for (Int32 i = 0; i < this.model.BoardWidth; i++)
                for (Int32 j = 0; j < this.model.BoardHeight; j++)
                {
                    switch (this.model[i, j])
                    {
                        case Player.PlayerO:
                            this.buttonBoard[i, j].BackColor = Color.Yellow;
                            this.buttonBoard[i, j].Text = "O";
                            break;
                        case Player.PlayerX:
                            this.buttonBoard[i, j].BackColor = Color.Orange;
                            this.buttonBoard[i, j].Text = "X";
                            break;
                        case Player.NoPlayer:
                            this.buttonBoard[i, j].BackColor = Color.White;
                            this.buttonBoard[i, j].Text = String.Empty;
                            break;
                    }
                }
        }

        #endregion

        #region Model event handlers

        /// <summary>
        /// Játék megnyerésének eseménykezelése.
        /// </summary>
        private void Model_GameWon(object sender, GameWonEventArgs e)
        {
            switch (e.Player)
            {
                case Player.PlayerO:
                    MessageBox.Show("A kör játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    break;
                case Player.PlayerX:
                    MessageBox.Show("A kereszt játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    break;
            }
            this.model.NewGame();
            this.UpdateBoard();
        }

        /// <summary>
        /// Játék végének eseménykezelése.
        /// </summary>
        private void Model_GameOver(object sender, EventArgs e)
        {
            MessageBox.Show("Döntetlen játék!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            this.model.NewGame();
            this.UpdateBoard();
        }

        /// <summary>
        /// Modell mezőváltozásának eseménykezelése.
        /// </summary>
        private void Model_FieldChanged(object sender, FieldChangedEventArgs e)
        {
            switch (e.Player)
            {
                case Player.PlayerO:
                    this.buttonBoard[e.X, e.Y].BackColor = Color.Yellow;
                    this.buttonBoard[e.X, e.Y].Text = "O";
                    break;
                case Player.PlayerX:
                    this.buttonBoard[e.X, e.Y].BackColor = Color.Orange;
                    this.buttonBoard[e.X, e.Y].Text = "X";
                    break;
            }
        }

        #endregion

        #region Form event handlers

        /// <summary>
        /// Ablak méretváltásának eseménykezelése.
        /// </summary>
        private void TicTacToeForm_SizeChanged(object sender, EventArgs e)
        {
            for (Int32 i = 0; i < this.model.BoardWidth; i++)
                for (Int32 j = 0; j < this.model.BoardHeight; j++)
                {
                    // a gombok szövegméretét állítjuk be az új mérethez arányosan
                    this.buttonBoard[i, j].Font = new Font(FontFamily.GenericSansSerif, Height / 10, FontStyle.Bold);
                }
        }

        #endregion

        #region Grid event handlers

        /// <summary>
        /// Gombrács eseménykezelője.
        /// </summary>
        private void ButtonBoard_Click(object sender, EventArgs e)
        {
            // a TabIndex-ből megkapjuk a sort és oszlopot
            Int32 x = ((sender as Button).TabIndex - 100) / this.model.BoardWidth;
            Int32 y = ((sender as Button).TabIndex - 100) % this.model.BoardWidth;

            try
            {
                this.model.StepGame(x, y); // lépés a játékban
            }
            catch { }
        }

        #endregion

        #region Menu event handlers

        /// <summary>
        /// Új játék menüpontjának eseménykezelője.
        /// </summary>
        private void MenuGameNew_Click(object sender, EventArgs e)
        {
            this.model.NewGame();
            this.UpdateBoard();
        }
        /// <summary>
        /// Játék betöltés menüpontjának eseménykezelője.
        /// </summary>
        private void MenuGameLoad_Click(object sender, EventArgs e)
        {
            if (this.openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.model.LoadGame(openFileDialog.FileName);
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a betöltés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        /// <summary>
        /// Játék mentés menüpontjának eseménykezelője.
        /// </summary>
        private void MenuGameSave_Click(object sender, EventArgs e)
        {
            if (this.saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.model.SaveGame(saveFileDialog.FileName);
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a mentés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        /// <summary>
        /// Kilépés menüpontjának eseménykezelője.
        /// </summary>
        private void MenuGameExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion
    }
}
